from typing import Any, Dict, List, Literal, Optional, Union

from pydantic import BaseModel, Field


class ChatImageOutput(BaseModel):
    image: str = Field(..., description="A base64 image")


class ChatVideoOutput(BaseModel):
    video: str = Field(..., description="A base64 video file")


class ChatSoundOutput(BaseModel):
    sound: str = Field(..., description="A base64 sound file")


class MarkdownParagraph(BaseModel):
    candidates: List[str] = Field(
        ..., description="", min_length=3, max_length=3
    )


class ChatMarkdownOutput(BaseModel):
    markdown: Union[str, MarkdownParagraph] = Field(...)


class ChatOOXMLOutput(BaseModel):
    document: str = Field(...)
    type: Literal["pptx", "docx"]


class ChatMessageOutput(BaseModel):
    chat_message: str = Field(...)


class ChatStructuredOutput(BaseModel):
    output: Dict[str, Any]


class ChatOutput(BaseModel):
    message: ChatMessageOutput = Field(...)
    image: Optional[ChatImageOutput] = Field(None)
    markdown: Optional[ChatMarkdownOutput] = Field(None)
    ooxml: Optional[ChatOOXMLOutput] = Field(None)
    sound: Optional[ChatSoundOutput] = Field(None)
    video: Optional[ChatVideoOutput] = Field(None)



from typing import ByteString, Optional, Union

from pydantic import BaseModel, Field


class ChatTextInput(BaseModel):
    text_message: str = Field(...)


class ChatFileInput(BaseModel):
    text_message: Optional[str] = Field(None)
    file: ByteString = Field(...)


class ChatCanvanReferenceInput(BaseModel):
    text_message: str = Field(...)
    start_position: int = Field(...)
    end_position: int = Field(...)


class ChatInput(BaseModel):
    input_string: Union[ChatTextInput, ChatFileInput, ChatCanvanReferenceInput]



from typing import Callable
from fastapi import FastAPI


class BirdieAPI(FastAPI):
    def __init__(self,
                 init_func: Callable,
                 interact_func: Callable,
                 input_func: Callable,
                 post_process_func: Callable,
                 **kwargs
                 ):
        super().__init__(**kwargs)
        self.add_api_route("/initialize", init_func, methods=["POST"])
        self.add_api_route("/interact", interact_func, methods=["POST"])
        self.add_api_route("/input", input_func, methods=["GET"])
        self.add_api_route(
            "/post-process",
            post_process_func,
            methods=["POST"]
        )
